﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.IO;
using System.Net.Sockets;
using System.Net;

namespace HomeMediaCenterPlayer
{
    public class UDPStream : Stream
    {
        private Socket socket;
        private readonly int port;

        public UDPStream(IPAddress localAddress, IPAddress broadcastAddress, int port)
        {
            this.port = port;
            this.socket = new Socket(AddressFamily.InterNetwork, SocketType.Dgram, ProtocolType.Udp);
            this.socket.SetSocketOption(SocketOptionLevel.Socket, SocketOptionName.ReuseAddress, 1);
            this.socket.SetSocketOption(SocketOptionLevel.Socket, SocketOptionName.Broadcast, 1);
            this.socket.SetSocketOption(SocketOptionLevel.IP, SocketOptionName.AddMembership,
                new MulticastOption(broadcastAddress, localAddress));
            this.socket.Bind(new IPEndPoint(localAddress, this.port));
        }

        public override bool CanRead
        {
            get { return true; }
        }

        public override bool CanSeek
        {
            get { return false; }
        }

        public override bool CanWrite
        {
            get { return true; }
        }

        public override void Flush()
        {
        }

        public override long Length
        {
            get { throw new NotImplementedException(); }
        }

        public override long Position
        {
            get { throw new NotImplementedException(); }
            set { throw new NotImplementedException(); }
        }

        public override int Read(byte[] buffer, int offset, int count)
        {
            return this.socket.Receive(buffer, offset, count, SocketFlags.None);
        }

        public override long Seek(long offset, SeekOrigin origin)
        {
            throw new NotImplementedException();
        }

        public override void SetLength(long value)
        {
            throw new NotImplementedException();
        }

        public override void Write(byte[] buffer, int offset, int count)
        {
            while (count > 0)
            {
                int send = Math.Min(count, this.socket.SendBufferSize);
                this.socket.SendTo(buffer, offset, send, SocketFlags.None, new IPEndPoint(IPAddress.Broadcast, this.port));
                offset += send;
                count -= send;
            }
        }

        protected override void Dispose(bool disposing)
        {
            base.Dispose(disposing);

            this.socket.Dispose();
        }
    }
}
